#include <Engine/Level/Scene.h>
#include <Engine/Entity/Camera.h>

bool Engine::Scene::EntityListContains(const Entity* entity) const {
    for (auto* e : entity_list)
        if (e == entity)
            return true;
    return false;
}

bool Engine::Scene::FixedListContains(const Fixed* fixed) const {
    for (auto* f : fixed_list)
        if (f == fixed)
            return true;
    return false;
}

size_t Engine::Scene::FixedCount() const {
    return fixed_list.size();
}

size_t Engine::Scene::EntityCount() const {
    return entity_list.size();
}

void Engine::Scene::Update() {
    if (active_camera)
        active_camera->Update();

    for (auto& e : entity_list)
        if (auto* c = dynamic_cast<Camera*>(e); c == nullptr)
            e->Update();
}

void Engine::Scene::Render(RenderTarget* render_target) {
    for (auto& f : fixed_list)
        if (f->Enabled())
            f->Render();

    // TODO Render order. In this system it's not possible for child entities to be rendered before the parent.

    J2D::Begin(render_target, true);

    if (active_camera)
        active_camera->Render();

    for (auto& e : entity_list)
        if (auto* r = dynamic_cast<Renderable*>(e))
            if (auto* c = dynamic_cast<Camera*>(r); c == nullptr)
                r->Render();

    if (heads_up_display)
        heads_up_display->Render();
    J2D::End();
}

Engine::Scene::~Scene() {
    for (auto* f : fixed_list)
        delete f;

    for (auto* e : entity_list)
        delete e;

    delete heads_up_display;
    delete active_camera;
}

void Engine::Scene::Unload() {
    for (auto* f : fixed_list)
        delete f;

    for (auto* e : entity_list)
        delete e;

    delete heads_up_display;
}

void Engine::Scene::AppendEntity(Entity* entity) {
    if (!EntityListContains(entity))
        entity_list.push_back(entity);
}

void Engine::Scene::AppendFixed(Fixed* fixed) {
    if (!FixedListContains(fixed))
        fixed_list.push_back(fixed);
}

void Engine::Scene::DestroyEntity(Entity *entity) {
    auto it = std::find(entity_list.begin(), entity_list.end(), entity);
    if (it != entity_list.end())
        delete *it, entity_list.erase(it);
}

void Engine::Scene::DestroyFixed(Fixed* fixed) {
    auto it = std::find(fixed_list.begin(), fixed_list.end(), fixed);
    if (it != fixed_list.end())
        delete *it, fixed_list.erase(it);
}

void Engine::Scene::RemoveEntity(Entity* entity) {
    auto it = std::find(entity_list.begin(), entity_list.end(), entity);
    if (it != entity_list.end())
        entity_list.erase(it);
}


void Engine::Scene::RemoveFixed(Fixed* fixed) {
    auto it = std::find(fixed_list.begin(), fixed_list.end(), fixed);
    if (it != fixed_list.end())
        fixed_list.erase(it);
}

std::string Engine::Scene::GetName() const {
    return name;
}

Engine::Camera* Engine::Scene::GetActiveCamera() const {
    return active_camera;
}

